import { createSignal } from 'solid-js';
import { Signaled } from './signaled';
export class SignaledIterator extends Signaled {
    constructor(iterator, initialValue) {
        let unsubscribe = () => { };
        const unsubscribePromise = new Promise((resolve) => (unsubscribe = () => resolve({ done: true, value: undefined })));
        const [accessor, setter] = createSignal(initialValue, { equals: false });
        const done = new Promise(async (resolve) => {
            while (true) {
                const { done, value } = await Promise.race([iterator.next(), unsubscribePromise]);
                if (done)
                    break;
                setter(() => value);
            }
            resolve();
        });
        super(accessor, undefined);
        this.done = done;
        this.unsubscribe = unsubscribe;
    }
    static fromIterator(iterator, initialValue) {
        return new SignaledIterator(iterator, initialValue);
    }
    static fromIterable(iterable, initialValue) {
        const iterator = iterable[Symbol.asyncIterator]();
        return new SignaledIterator(iterator, initialValue);
    }
}
export const createIterator = (iterator, initialValue) => SignaledIterator.fromIterator(iterator, initialValue);
export const createIterable = (iterable, initialValue) => SignaledIterator.fromIterable(iterable, initialValue);
